###########################################################################
# date : Jan 2024
# this is a demo for implementation of pGVEM
# Description: this file provides pGVEM algorithm for multidimensional partial credit model
# The Exploratory factor analysis is provided with a default rotation 'promax' with m=4
# The input include N x J matrix of response, where 0 is the minimal partial credit score.
# The latent dimension also need specifying.
# By default the SE will not be calculated. We implement the Bootstrap method in the function, where number of bootstrap samples is by default B=50.
# The return of the function contains the estimated a: a_hat, estimated b: b_hat, estimated correlation: sig_hat, mean of variational distribution: thet_hat,
# standard error of a: se_a, standard error of b: se_b, evidence lower bound on the estimation: elbo and the compuatation time: time.
# Auxiliary functions include 
#  generate_para: generate a set of true parameters with specified range of correlation and discrimination parameters drawn from uniform distribution,
#                 and the lsit of numbers of categoeis for all items needs to be specified.
#  generate_response: generate the response matrix with designated parameters.
#  elbo: calculating the evidence lower bound given in (3.5)


###########################################################################
library(MASS)
library(mvtnorm)
library(stats4)
library(lattice)
library(mirt)
library(gtools)
library(GPArotation);


path <- "your_path/demo_pGVEM"
setwd(path)
source(paste0(path, "functions.R"))


## Set test scale
N <- 500;
J <- 20;
K_list <- c(rep(3,10), rep(5,10));

D <- 3;

res1=generate_para(N, J , D, K_list ,aL = 0.5 , aU = 1)
resp=generate_response(res1$thet_true, res1$a_true, res1$b_true, K_list)
res2=gvemeva(resp, D, SE = TRUE, B = 50)
